# -*- coding: utf-8 -*-
"""
Created on Wed Nov  6 20:23:49 2019

@author: Lloyd
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import os
import platform
import time
import tensorflow as tf
# tf.executing_eagerly()

import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.simplefilter(action='ignore', category=Warning)
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3'  # try to ignore warnings

from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPool2D, Dropout
from tensorflow.keras import Model

tf.keras.backend.set_floatx('float32') # float64 eliminates warning but slows it down a lot
tf.keras.backend.clear_session()  # clear models from previous sessions
if 'model' in locals():   # delete model if abnormal exit from previous run
    del model
    
# Load and prepare the MNIST dataset.
time1 = time.time() # time at start of work

mnist = tf.keras.datasets.mnist

(x_train, y_train), (x_test, y_test) = mnist.load_data()
x_train, x_test = x_train / 255.0, x_test / 255.0

# Add a channels dimension
x_train = x_train[..., tf.newaxis]
x_test = x_test[..., tf.newaxis]

# Use tf.data to batch and shuffle the dataset:
train_ds = tf.data.Dataset.from_tensor_slices(
    (x_train, y_train)).shuffle(10000).batch(32)

test_ds = tf.data.Dataset.from_tensor_slices((x_test, y_test)).batch(32)

# Build the tf.keras model using the Keras model subclassing API:

class MyModel(Model):
  def __init__(self):
    super(MyModel, self).__init__()
    self.conv1 = Conv2D(32, 5, activation='relu')
    self.pool1 = MaxPool2D()
    self.conv2 = Conv2D(64, 5, activation='relu')
    self.pool2 = MaxPool2D()
    self.flatten = Flatten()
    self.d1 = Dense(1024, activation='relu')
    self.drop1 = Dropout(.5)
    self.d2 = Dense(10, activation='softmax')

  def call(self, x):
    x = self.conv1(x)
    x = self.pool1(x)
    x = self.conv2(x)
    x = self.pool2(x)
    x = self.flatten(x)
    x = self.d1(x)
    x = self.drop1(x)
    return self.d2(x)

# Create an instance of the model
model = MyModel()

# Choose an optimizer and loss function for training:

loss_object = tf.keras.losses.SparseCategoricalCrossentropy()

optimizer = tf.keras.optimizers.Adam()

# Select metrics to measure the loss and the accuracy of the model. These metrics accumulate the values over epochs and then print the overall result.

train_loss = tf.keras.metrics.Mean(name='train_loss')
train_accuracy = tf.keras.metrics.SparseCategoricalAccuracy(name='train_accuracy')

test_loss = tf.keras.metrics.Mean(name='test_loss')
test_accuracy = tf.keras.metrics.SparseCategoricalAccuracy(name='test_accuracy')

# Use tf.GradientTape to train the model:

@tf.function
def train_step(images, labels):
  with tf.GradientTape() as tape:
    predictions = model(images)
    loss = loss_object(labels, predictions)
  gradients = tape.gradient(loss, model.trainable_variables)
  optimizer.apply_gradients(zip(gradients, model.trainable_variables))

  train_loss(loss)
  train_accuracy(labels, predictions)

# Test the model:

@tf.function
def test_step(images, labels):
  predictions = model(images)
  t_loss = loss_object(labels, predictions)

  test_loss(t_loss)
  test_accuracy(labels, predictions)

EPOCHS = 5

# print("\nBegin Training")

for epoch in range(EPOCHS):
  for images, labels in train_ds:
    train_step(images, labels)

  for test_images, test_labels in test_ds:
    test_step(test_images, test_labels)

  time2 = time.time()
  runTime = time2 - time1

  if epoch==0:
    print("\nMNIST Deep Benchmark")
    print("Platform:  %s  %s  %s" % (platform.platform(),platform.machine(),platform.architecture()))
    print("           %s  %s" % (platform.node(),platform.processor()))
    print ("start time = %s\n" %  (time.asctime( time.localtime(time1) )))
      
  print("Epoch %2d, Loss: %7.4f, Accuracy: %5.2f %%, Test Loss: %7.4f, Test Accuracy: %5.2f %%, Time: %7.2f s" % (epoch+1,
                        train_loss.result(),
                        train_accuracy.result()*100,
                        test_loss.result(),
                        test_accuracy.result()*100,
                        runTime))

  # Reset the metrics for the next epoch
  train_loss.reset_states()
  train_accuracy.reset_states()
  test_loss.reset_states()
  test_accuracy.reset_states()

# Delete the instance of the model
del model
  
time2 = time.time()  
runTime = time2 - time1
print ("\nend time = %s    run time = %7.2f s\n\n" %  (time.asctime( time.localtime(time2) ),runTime))

